package com.diagbot.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.*;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.DoctorAverageStatisticsVO;
import com.diagbot.vo.GetDetailRecordListPageVO;
import com.diagbot.vo.GetDoctorDetailPageVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;


/**
 * @Description:控制台（医生）统计相关处理
 * @Author:songxl
 * @date 2022/4/8 13:20
 */
@Component
public class ConsoleByDoctorFacade {
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    @Autowired
    QcresultInfoFacade qcresultInfoFacade;


    /**
     * 医生质控平均分页面统计
     *
     * @param doctorAverageStatisticsVO
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.diagbot.dto.DoctorAverageStatisticsDTO>
     */
    public IPage<DoctorAverageStatisticsDTO> doctorAverageStatistics(DoctorAverageStatisticsVO doctorAverageStatisticsVO) {
        doctorAverageStatisticsVO.setHospitalId(SysUserUtils.getCurrentHospitalID());
        //替换
        transformParam(doctorAverageStatisticsVO);
        return behospitalInfoFacade.getBaseMapper().doctorAverageStatistics(doctorAverageStatisticsVO);
    }

    /**
     * 参数替换
     *
     * @param doctorAverageStatisticsVO
     * @Return void
     */
    private void transformParam(DoctorAverageStatisticsVO doctorAverageStatisticsVO) {

        if ("-".equals(doctorAverageStatisticsVO.getDeptId())) {
            doctorAverageStatisticsVO.setDeptId("-0");
        }
        if ("-".equals(doctorAverageStatisticsVO.getDeptName())) {
            doctorAverageStatisticsVO.setDeptName("-0");
        }
        if ("-".equals(doctorAverageStatisticsVO.getDoctorId())) {
            doctorAverageStatisticsVO.setDoctorId("-0");
        }
        if ("-".equals(doctorAverageStatisticsVO.getDoctorName())) {
            doctorAverageStatisticsVO.setDoctorName("-0");
        }
    }

    /**
     * 获取缺陷列表
     *
     * @param getDoctorDetailPageVO
     * @return
     */
    public IPage<GetDoctorDetailPageDTO> getDoctorDetailPage(GetDoctorDetailPageVO getDoctorDetailPageVO) {
        if ("-".equals(getDoctorDetailPageVO.getDeptId())) {
            getDoctorDetailPageVO.setDeptId("-0");
        }
        if ("-".equals(getDoctorDetailPageVO.getDeptName())) {
            getDoctorDetailPageVO.setDeptName("-0");
        }
        if ("-".equals(getDoctorDetailPageVO.getDoctorId())) {
            getDoctorDetailPageVO.setDoctorId("-0");
        }
        if ("-".equals(getDoctorDetailPageVO.getDoctorName())) {
            getDoctorDetailPageVO.setDoctorName("-0");
        }
        getDoctorDetailPageVO.setHospitalId(SysUserUtils.getCurrentHospitalID());
        IPage<GetDoctorDetailPageDTO> detailPageList = behospitalInfoFacade.getBaseMapper().getDoctorDetailPage(getDoctorDetailPageVO);
        return detailPageList;
    }

    /**
     * 获取缺陷数量对应的病历
     *
     * @param getDetailRecordListPageVO
     * @return
     */
    public IPage<GetDetailRecordListPageDTO> getDetailRecordListPage(GetDetailRecordListPageVO getDetailRecordListPageVO) {
        if ("-".equals(getDetailRecordListPageVO.getDeptId())) {
            getDetailRecordListPageVO.setDeptId("-0");
        }
        if ("-".equals(getDetailRecordListPageVO.getDeptName())) {
            getDetailRecordListPageVO.setDeptName("-0");
        }
        if ("-".equals(getDetailRecordListPageVO.getDoctorId())) {
            getDetailRecordListPageVO.setDoctorId("-0");
        }
        if ("-".equals(getDetailRecordListPageVO.getDoctorName())) {
            getDetailRecordListPageVO.setDoctorName("-0");
        }
        getDetailRecordListPageVO.setHospitalId(SysUserUtils.getCurrentHospitalID());
        IPage<GetDetailRecordListPageDTO> detailPageList = behospitalInfoFacade.getBaseMapper().getDetailRecordListPage(getDetailRecordListPageVO);
        return detailPageList;
    }

    /**
     * 导出缺陷数量对应的病历
     *
     * @param getDetailRecordListPageVO
     * @return
     */
    public List<ExportExcelDTO> getDetailRecordListExport(GetDetailRecordListPageVO getDetailRecordListPageVO) {
        if ("-".equals(getDetailRecordListPageVO.getDeptId())) {
            getDetailRecordListPageVO.setDeptId("-0");
        }
        if ("-".equals(getDetailRecordListPageVO.getDeptName())) {
            getDetailRecordListPageVO.setDeptName("-0");
        }
        if ("-".equals(getDetailRecordListPageVO.getDoctorId())) {
            getDetailRecordListPageVO.setDoctorId("-0");
        }
        if ("-".equals(getDetailRecordListPageVO.getDoctorName())) {
            getDetailRecordListPageVO.setDoctorName("-0");
        }
        getDetailRecordListPageVO.setHospitalId(SysUserUtils.getCurrentHospitalID());
        List<ExportExcelDTO> detailPageList = behospitalInfoFacade.getBaseMapper().getDetailRecordListExport(getDetailRecordListPageVO);
        return detailPageList;
    }

}
