package com.diagbot.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.*;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.vo.*;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

import javax.validation.Valid;

/**
 * <p>
 * 住院病历信息 Mapper 接口
 * </p>
 *
 * @author zhoutg
 * @time: 2024-04-13
 */
public interface BehospitalInfoMapper extends BaseMapper<BehospitalInfo> {

    public IPage<BehospitalInfoDTO> getPage(BehospitalPageVO behospitalPageVO);

    public List<MsgDTO> getMsg(AnalyzeVO analyzeVO);

    public List<MsgDTO> getMsgDetail(AnalyzeVO analyzeVO);

    public List<MsgDTO> getMsgByEntryCode(AnalyzeCodeVO analyzeCodeVO);

    /**
     * 各科室缺陷占比-全院-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> entryByDept(FilterVO filterVO);

    /**
     * 质控结果缺陷总数查询
     *
     * @param filterVO
     * @return
     */
    public int getTotleResultNum(FilterVO filterVO);

    public List<BehospitalInfo> getNoGrade(TaskVO taskVO);

    /**
     * 出院总人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int leaveHosCount(FilterVO filterVO);

    /**
     *  医生质控平均分页面统计
     *
     * @param doctorAverageStatisticsVO
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.diagbot.dto.DoctorAverageStatisticsDTO>
     */
    IPage<DoctorAverageStatisticsDTO> doctorAverageStatistics(@Param("doctorAverageStatisticsVO") DoctorAverageStatisticsVO doctorAverageStatisticsVO);

    /**
     * 新生儿出院人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int newBornCount(FilterVO filterVO);

    /**
     * 死亡人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int deathCount(FilterVO filterVO);

    /**
     * 手术人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int operationCount(FilterVO filterVO);


    /**
     * 入院总人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<AdmissionCountDTO> admissionHosCountByDept(FilterVO filterVO);

    /**
     * 新生儿出院人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> newBornCountByDept(FilterVO filterVO);

    /**
     * 死亡人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> deathCountByDept(FilterVO filterVO);

    /**
     * 手术人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> operationCountByDept(FilterVO filterVO);

    /**
     * 各科室缺陷占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> resultStatisticsByDeptPage(@Param("filterPageVO") FilterPageVO filterPageVO);

    /**
     * 病案首页合格率占比
     *
     * @param filterOrderVO
     * @return
     */
    public List<HomePageNumDTO> homePageLevelStatistics(FilterOrderVO filterOrderVO);

    /**
     * 各科室缺陷占比（组合）
     *
     * @param filterOrderVO
     * @return
     */
    public List<LevelStatisticsDTO> levelStatistics(FilterOrderVO filterOrderVO);

    /**
     * 病历质控报表-扣分数据
     *
     * @param caseScoreVO
     * @return
     */
    public List<CaseScoreDTO> getQualityCaseSorce(CaseScoreVO caseScoreVO);

    /**
     * 病历质控报表科室-扣分数据
     *
     * @param caseScoreVO
     * @return
     */
    public List<CaseScoreDTO> getQualityCaseSorceByDept(CaseScoreVO caseScoreVO);


    /**
     * 病历质控报表
     *
     * @param behospitalPageVO
     * @return
     */
    public IPage<QualityControlDTO> getQualityControl(BehospitalPageVO behospitalPageVO);

    /**
     * 科室缺陷占比-科室（分页）
     *
     * @param filterPageByDeptVO
     * @return
     */
    public IPage<DeptNumDTO> resultStatisticsByDeptAndDoctorPage(@Param("filterPageByDeptVO") FilterPageByDeptVO filterPageByDeptVO);


    /**
     * 病案首页合格/不合格数
     *
     * @param qcResultPageVO
     * @return
     */
    public IPage<QcResultShortDeptDTO> getIsGoodLevelByDept(QcResultPageVO qcResultPageVO);


    List<ExportExcelDTO> exportExcel();

    /**
     * 质控评分导出到excel
     *
     * @param exportQcresultVO
     * @return
     */
    public List<ExportExcelDTO> exportQcresult(ExportQcresultVO exportQcresultVO);

    /**
     * 条目缺陷质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> qcResultShortPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 根据两个主键-批量修改
     *
     * @param list
     */
    public void updateBatchByKey(List<BehospitalInfo> list);

    /**
     * 关键条目缺陷占比统计
     *
     * @param entryStatisticsVO
     * @return
     */
    public List<EntryStatisticsDTO> entryStatistics(EntryStatisticsVO entryStatisticsVO);

    /**
     * 三级查房统计报表
     *
     * @param entryStatisticsVO
     * @return
     */
    public List<SjcfStatisticsDTO> sjcfStatistics(@Valid EntryStatisticsVO entryStatisticsVO);

    /**
     * 非医嘱离院病人记录
     *
     * @param filterVO
     * @return
     */
    public int nonAdviceCount(FilterVO filterVO);

    /**
     * 31天内再入院记录
     *
     * @param filterVO
     * @return
     */
    public int get31DaysBehospitalCount(FilterVO filterVO);

    /**
     * 病案首页质控病历数统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    public HomePageNumDTO homePageQcPercent(FilterVO filterVO);

    /**
     * 病案首页完整率统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    public HomePageNumDTO hmEmptyEntryPercent(FilterVO filterVO);

    /**
     * 离院病人质控评分详情页
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> leaveHosMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 改善率统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    public HomePageImproveDTO homePageImproveCount(FilterVO filterVO);

    /**
     * 质控核查统计（内页）
     *
     * @param filterOrderVO
     * @return
     */
    public List<HomePageImproveDTO> qcCheckStatistics(FilterOrderVO filterOrderVO);


    /**
     * 离院病人质控评分详情页导出到excel
     *
     * @param qcResultShortPageVO
     * @return
     */
    public List<ExportExcelDTO> leaveHosMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);


    /**
     * 条目缺陷质控评分详情页导出到excel
     *
     * @param qcResultShortPageVO
     * @return
     */
    public List<ExportExcelDTO> qcResultShortPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 未整改病历统计
     *
     * @param filterUnModifyMRVO
     * @return
     */
    public List<UnModifyMRDetailDTO> unModifyMRStatistics(FilterUnModifyMRVO filterUnModifyMRVO);

    /**
     * 未整改病历质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> unModifyMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 不合格/合格数病历号（内页）
     *
     * @param qcResultPageVO
     * @return
     */
    public IPage<QcResultShortDTO> badLevelPage(@Param("qcResultPageVO") QcResultPageVO qcResultPageVO);


    /**
     * 未整改病历缺陷评分详情页导出
     *
     * @param qcResultShortPageVO
     * @return
     */

    public List<ExportExcelDTO> unModifyMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);


    /**
     * 病案首页不合格/合格数病历详情页导出
     *
     * @param qcResultShortPageVO
     * @return
     */

    public List<ExportExcelDTO> badLevelPagePageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 31天再入院详情页
     *
     * @param reBeHosPageVO
     * @return
     */
    public IPage<ReBeHosMergeDTO> reHos31DaysPage(@Param("reBeHosPageVO") ReBeHosPageVO reBeHosPageVO);

    /**
     * 入院人数统计-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> beHosCount(FilterVO filterVO);

    /**
     * 单条条目缺陷统计-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> casesEntryStatisticsById(FilterVO filterVO);


    /**
     * 病案首页改善率质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> hmImproveMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 病案首页改善率质控评分页导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    public List<ExportExcelDTO> hmImproveMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 病案管理指标获取基础数据
     *
     * @param timeVo
     * @return
     */
    public Map<String,Object> getBaseIndex( MedIndexFilterVO timeVo);

    /**
     * 病案管理规则质控参数查询
     *
     * @param filterVO
     * @return
     */
    public Map<String,Object> getCountByEntry( FilterVO filterVO);

    /**
     * 病案管理规则质控病历参数查询
     *
     * @param filterVO
     * @return
     */

    public  List<MedManageParamsDTO> getMedManageParams( FilterVO filterVO);


    /**
     * 质控核查质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> qcCheckMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 质控核查质控评分页导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    public List<ExportExcelDTO> qcCheckMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO);

    /**
     * 缺陷详情（分页）(根据医生)
     *
     * @param getDoctorDetailPageVO
     * @return
     */
    public IPage<GetDoctorDetailPageDTO> getDoctorDetailPage(@Param("getDoctorDetailPageVO") GetDoctorDetailPageVO getDoctorDetailPageVO);

    /**
     * 获取缺陷数量对应的病历
     *
     * @param getDetailRecordListPageVO
     * @return
     */
    public IPage<GetDetailRecordListPageDTO> getDetailRecordListPage(@Param("getDetailRecordListPageVO") GetDetailRecordListPageVO getDetailRecordListPageVO);

    /**
     * 导出缺陷数量对应的病历
     *
     * @param getDetailRecordListPageVO
     * @return
     */
    public List<ExportExcelDTO> getDetailRecordListExport(@Param("getDetailRecordListPageVO") GetDetailRecordListPageVO getDetailRecordListPageVO);

}

