package com.diagbot.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.diagbot.dto.*;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.mapper.BehospitalInfoMapper;
import com.diagbot.service.BehospitalInfoService;
import com.diagbot.vo.*;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Service;

import java.util.*;

import javax.validation.Valid;

/**
 * <p>
 * 住院病历信息 服务实现类
 * </p>
 *
 * @author zhoutg
 * @time: 2024-04-13
 */
@Service
public class BehospitalInfoServiceImpl extends ServiceImpl<BehospitalInfoMapper, BehospitalInfo> implements BehospitalInfoService {

    @Override
    public IPage<BehospitalInfoDTO> getPage(BehospitalPageVO behospitalPageVO) {
        return baseMapper.getPage(behospitalPageVO);
    }

    @Override
    public List<MsgDTO> getMsg(AnalyzeVO analyzeVO) {
        return baseMapper.getMsg(analyzeVO);
    }
    @Override
    public List<MsgDTO> getMsgDetail(AnalyzeVO analyzeVO) {
        return baseMapper.getMsgDetail(analyzeVO);
    }
    @Override
    public List<MsgDTO> getMsgByEntryCode(AnalyzeCodeVO analyzeCodeVO) {
        return baseMapper.getMsgByEntryCode(analyzeCodeVO);
    }

    /**
     * 各科室缺陷占比-全院-首页
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<NumDTO> entryByDept(FilterVO filterVO) {
        return baseMapper.entryByDept(filterVO);
    }

    @Override
    public List<BehospitalInfo> getNoGrade(TaskVO taskVO) {
        return baseMapper.getNoGrade(taskVO);
    }

    /**
     * 出院总人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int leaveHosCount(FilterVO filterVO) {
        return baseMapper.leaveHosCount(filterVO);
    }

    /**
     * 新生儿出院人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int newBornCount(FilterVO filterVO) {
        return baseMapper.newBornCount(filterVO);
    }

    /**
     * 死亡人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int deathCount(FilterVO filterVO) {
        return baseMapper.deathCount(filterVO);
    }

    /**
     * 手术人数统计-全院-首页
     *
     * @param filterVO
     * @return
     */
    public int operationCount(FilterVO filterVO) {
        return baseMapper.operationCount(filterVO);
    }

    /**
     * 入院总人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<AdmissionCountDTO> admissionHosCountByDept(FilterVO filterVO) {
        return baseMapper.admissionHosCountByDept(filterVO);
    }

    /**
     * 新生儿出院人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> newBornCountByDept(FilterVO filterVO) {
        return baseMapper.newBornCountByDept(filterVO);
    }

    /**
     * 死亡人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> deathCountByDept(FilterVO filterVO) {
        return baseMapper.deathCountByDept(filterVO);
    }

    /**
     * 手术人数统计-科室-首页
     *
     * @param filterVO
     * @return
     */
    public List<NumDTO> operationCountByDept(FilterVO filterVO) {
        return baseMapper.operationCountByDept(filterVO);
    }

    /**
     * 病案首页合格/不合格数-科室
     *
     * @param qcResultPageVO
     * @return
     */
    public IPage<QcResultShortDeptDTO> getOrGoodLevelByDept(QcResultPageVO qcResultPageVO) {
        return  baseMapper.getIsGoodLevelByDept(qcResultPageVO);
    }

    /**
     * 各科室缺陷占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    @Override
    public IPage<NumDTO> resultStatisticsByDeptPage(@Param("filterPageVO") FilterPageVO filterPageVO) {
        return baseMapper.resultStatisticsByDeptPage(filterPageVO);
    }

    /**
     * 各科室缺陷占比（组合）
     *
     * @param filterOrderVO
     * @return
     */
    @Override
    public List<LevelStatisticsDTO> levelStatistics(FilterOrderVO filterOrderVO) {
        return baseMapper.levelStatistics(filterOrderVO);
    }

//    @Override
//    public List<HomePageNumDTO> homePageLevelStatistics(FilterOrderVO filterOrderVO) {
//        return Collections.emptyList();
//    }
    /**
     * 病案首页合格率占比
     *
     * @param filterOrderVO
     * @return
     */
    @Override
    public List<HomePageNumDTO> homePageLevelStatistics(FilterOrderVO filterOrderVO) {
        return baseMapper.homePageLevelStatistics(filterOrderVO);
    }

    /**
     * 病历质控报表-扣分数据
     *
     * @param caseScoreVO
     * @return
     */
    @Override
    public List<CaseScoreDTO> getQualityCaseSorce(CaseScoreVO caseScoreVO) {
        return baseMapper.getQualityCaseSorce(caseScoreVO);
    }

    /**
     * 病历质控报表科室-扣分数据
     *
     * @param caseScoreVO
     * @return
     */
    @Override
    public List<CaseScoreDTO> getQualityCaseSorceByDept(CaseScoreVO caseScoreVO) {
        return baseMapper.getQualityCaseSorceByDept(caseScoreVO);
    }

    /**
     * 病历质控报表
     *
     * @param behospitalPageVO
     * @return
     */
    @Override
    public IPage<QualityControlDTO> getQualityControl(BehospitalPageVO behospitalPageVO) {
        return baseMapper.getQualityControl(behospitalPageVO);
    }

    /**
     * 缺陷明细-科室（分页）
     *
     * @param filterPageByDeptVO
     * @return
     */
    public IPage<DeptNumDTO> resultStatisticsByDeptAndDoctorPage(@Param("filterPageByDeptVO") FilterPageByDeptVO filterPageByDeptVO) {
        return baseMapper.resultStatisticsByDeptAndDoctorPage(filterPageByDeptVO);
    }


    @Override
    public List<ExportExcelDTO> exportExcel() {
        return baseMapper.exportExcel();
    }

    /**
     * 质控评分导出到excel
     *
     * @param exportQcresultVO
     * @return
     */
    public List<ExportExcelDTO> exportQcresult(ExportQcresultVO exportQcresultVO) {
        return baseMapper.exportQcresult(exportQcresultVO);
    }


    /**
     * 条目缺陷质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    public IPage<QcResultShortDTO> qcResultShortPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.qcResultShortPage(qcResultShortPageVO);
    }

    /**
     * 根据编码、医院编码批量更新
     *
     * @param list
     */
    public void updateBatchByKey(List<BehospitalInfo> list) {
        this.baseMapper.updateBatchByKey(list);
    }

    /**
     * 关键条目缺陷占比统计
     *
     * @param entryStatisticsVO
     * @return
     */
    @Override
    public List<EntryStatisticsDTO> entryStatistics(EntryStatisticsVO entryStatisticsVO) {
        return baseMapper.entryStatistics(entryStatisticsVO);
    }

    public List<SjcfStatisticsDTO> sjcfStatistics(@Valid EntryStatisticsVO entryStatisticsVO) {
    	 return baseMapper.sjcfStatistics(entryStatisticsVO);
	}


    /**
     * 非医嘱离院病人记录
     *
     * @param filterVO
     * @return
     */
    @Override
    public int nonAdviceCount(FilterVO filterVO) {
        return baseMapper.nonAdviceCount(filterVO);
    }

    /**
     * 31天内再入院记录
     *
     * @param filterVO
     * @return
     */
    @Override
    public int get31DaysBehospitalCount(FilterVO filterVO) {
        return baseMapper.get31DaysBehospitalCount(filterVO);
    }

    /**
     * 病案首页质控病历数统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    @Override
    public HomePageNumDTO homePageQcPercent(FilterVO filterVO) {
        return baseMapper.homePageQcPercent(filterVO);
    }

    /**
     * 病案首页完整率统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    @Override
    public HomePageNumDTO hmEmptyEntryPercent(FilterVO filterVO) {
        return baseMapper.hmEmptyEntryPercent(filterVO);
    }

    /**
     * 离院病人质控评分详情页
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public IPage<QcResultShortDTO> leaveHosMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        IPage<QcResultShortDTO> qcResultShortDTOIPage = baseMapper.leaveHosMRPage(qcResultShortPageVO);
        return qcResultShortDTOIPage;
    }

    /**
     * 改善率统计-全院（首页）
     *
     * @param filterVO
     * @return
     */
    @Override
    public HomePageImproveDTO homePageImproveCount(FilterVO filterVO) {
        return baseMapper.homePageImproveCount(filterVO);
    }

    /**
     * 质控核查统计（内页）
     *
     * @param filterOrderVO
     * @return
     */
    @Override
    public List<HomePageImproveDTO> qcCheckStatistics(FilterOrderVO filterOrderVO) {
        return baseMapper.qcCheckStatistics(filterOrderVO);
    }

    /**
     * 离院病人质控评分详情页导出到excel
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public List<ExportExcelDTO> leaveHosMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.leaveHosMRPageExport(qcResultShortPageVO);
    }

    /**
     * 条目缺陷质控评分详情页导出到excel
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public List<ExportExcelDTO> qcResultShortPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.qcResultShortPageExport(qcResultShortPageVO);
    }

    /**
     * 未整改病历统计
     *
     * @param filterUnModifyMRVO
     * @return
     */
    @Override
    public List<UnModifyMRDetailDTO> unModifyMRStatistics(FilterUnModifyMRVO filterUnModifyMRVO) {
        return baseMapper.unModifyMRStatistics(filterUnModifyMRVO);
    }


    /**
     * 未整改病历质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public IPage<QcResultShortDTO> unModifyMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.unModifyMRPage(qcResultShortPageVO);
    }

    /**
     * 不合格/合格数病历号（内页）
     *
     * @param qcResultPageVO
     * @return
     */
    @Override
    public IPage<QcResultShortDTO> badLevelPage(@Param("qcResultPageVO") QcResultPageVO qcResultPageVO) {
        return baseMapper.badLevelPage(qcResultPageVO);
    }

    /**
     * 未整改病历缺陷评分详情页导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public List<ExportExcelDTO> unModifyMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.unModifyMRPageExport((qcResultShortPageVO));
    }


    /**
     * 病案首页不合格/合格数病历详情页导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public List<ExportExcelDTO> badLevelPagePageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.badLevelPagePageExport((qcResultShortPageVO));
    }

    /**
     * 31天再入院详情页
     *
     * @param reBeHosPageVO
     * @return
     */
    @Override
    public IPage<ReBeHosMergeDTO> reHos31DaysPage(@Param("reBeHosPageVO") ReBeHosPageVO reBeHosPageVO) {
        return baseMapper.reHos31DaysPage(reBeHosPageVO);
    }

    /**
     * 入院人数统计-首页
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<NumDTO> beHosCount(FilterVO filterVO) {
        return baseMapper.beHosCount(filterVO);
    }

    /**
     * 单条条目缺陷统计-首页
     *
     * @param filterVO
     * @return
     */
    @Override
    public List<NumDTO> casesEntryStatisticsById(FilterVO filterVO) {
        return baseMapper.casesEntryStatisticsById(filterVO);
    }

    /**
     * 病案首页改善率质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public IPage<QcResultShortDTO> hmImproveMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.hmImproveMRPage(qcResultShortPageVO);
    }

    /**
     * 病案首页改善率质控评分页导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public List<ExportExcelDTO> hmImproveMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.hmImproveMRPageExport(qcResultShortPageVO);
    }

    /**
     * 病案管理规则质控病历参数查询
     *
     * @param filterVO
     * @return
     */
    @Override
    public  List<MedManageParamsDTO> getMedManageParams( FilterVO filterVO) {
        return baseMapper.getMedManageParams(filterVO);
    }

    /**
     * 质控核查质控评分页（内页）
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public IPage<QcResultShortDTO> qcCheckMRPage(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.qcCheckMRPage(qcResultShortPageVO);
    }


    /**
     * 质控核查质控评分页导出
     *
     * @param qcResultShortPageVO
     * @return
     */
    @Override
    public List<ExportExcelDTO> qcCheckMRPageExport(@Param("qcResultShortPageVO") QcResultShortPageVO qcResultShortPageVO) {
        return baseMapper.qcCheckMRPageExport(qcResultShortPageVO);
    }
}
